"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
require("codemirror/mode/markdown/markdown");
const react_1 = __importStar(require("react"));
require("./codemirror.less");
require("./index.less");
const SERVER_RENDERED = (typeof navigator === 'undefined' || global.PREVENT_CODEMIRROR_RENDER === true);
let cm;
if (!SERVER_RENDERED) {
    // tslint:disable-next-line: no-var-requires
    cm = require('codemirror');
}
class ReactCodeMirror extends react_1.Component {
    // public editor!: Doc | Editor | EditorFromTextArea | Editor;
    constructor(props) {
        super(props);
    }
    render() {
        return (react_1.default.createElement("textarea", { ref: (instance) => this.textarea = instance }));
    }
    componentWillMount() {
        if (SERVER_RENDERED) {
            return;
        }
        if (this.props.editorWillMount) {
            this.props.editorWillMount();
        }
    }
    componentDidMount() {
        if (SERVER_RENDERED) {
            return;
        }
        const { options } = this.props;
        if (this.props.defineMode) {
            if (this.props.defineMode.name && this.props.defineMode.fn) {
                cm.defineMode(this.props.defineMode.name, this.props.defineMode.fn);
            }
        }
        const editorOption = Object.assign({ tabSize: 2, lineNumbers: true }, options, { mode: 'markdown' });
        // 生成codemirror实例
        this.editor = cm.fromTextArea(this.textarea, editorOption);
        this.renderCodeMirror(this.props);
    }
    componentWillReceiveProps(nextPros) {
        return __awaiter(this, void 0, void 0, function* () {
            this.renderCodeMirror(nextPros);
        });
    }
    // 将props中所有的事件处理函数映射并保存
    getEventHandleFromProps() {
        const propNames = Object.keys(this.props);
        const eventHandle = propNames.filter((prop) => {
            return /^on+/.test(prop);
        });
        const eventDict = {};
        eventHandle.forEach((ele) => {
            eventDict[ele] = ele.replace(/^on[A-Z]/g, s => s.slice(2).toLowerCase());
        });
        return eventDict;
    }
    renderCodeMirror(props) {
        const { value, width, height } = props;
        // 获取CodeMirror用于获取其中的一些常量
        // 事件处理映射
        const eventDict = this.getEventHandleFromProps();
        Object.keys(eventDict).forEach((event) => {
            const handle = this.props[event];
            this.editor.on(eventDict[event], handle);
        });
        // Init value
        this.editor.setValue(value || '');
        // this.editor.setOption(name, editorOption.mode);
        if (width || height) {
            // Setting size
            this.editor.setSize(width, height);
        }
    }
}
ReactCodeMirror.defaultProps = {
    height: '100%',
    options: {
        lineNumbers: true,
        mode: 'markdown',
        tabSize: 2,
    },
    value: '',
    width: '100%',
};
exports.default = ReactCodeMirror;
