"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const classnames_1 = __importDefault(require("classnames"));
const React = __importStar(require("react"));
const CodeMirror_1 = __importDefault(require("./components/CodeMirror"));
const PreviewMarkdown_1 = __importDefault(require("./components/PreviewMarkdown"));
const ToolBar_1 = __importDefault(require("./components/ToolBar"));
require("./index.less");
class MarkdownEditor extends React.PureComponent {
    constructor(props) {
        super(props);
        this.getInstance = (editor) => {
            if (editor) {
                this.CodeMirror = editor;
            }
        };
        this.onChange = (editor, data, value) => {
            const { onChange } = this.props;
            if (onChange) {
                if (this.preview) {
                    this.preview.updateSource(editor.getValue());
                }
                onChange(editor, data, value);
            }
        };
        this.onClick = (type) => {
            if (type === 'preview') {
                this.previewMarkdown();
                return;
            }
            const selection = this.CodeMirror.editor.getSelection();
            const pos = this.CodeMirror.editor.getCursor();
            let value = '';
            // 'header', 'strike', 'underline', 'olist', 'ulist', 'todo', 'link', 'image', 'quote', 'preview'
            if (type === 'bold') {
                value = selection ? `**${selection}**` : `****`;
                pos.ch = selection ? pos.ch : pos.ch + 2;
            }
            if (type === 'italic') {
                value = selection ? `*${selection}*` : `**`;
                pos.ch = selection ? pos.ch : pos.ch + 1;
            }
            if (type === 'header') {
                value = selection ? `# ${selection}` : '# ';
                pos.ch = selection ? pos.ch : pos.ch + 2;
            }
            if (type === 'strike') {
                value = selection ? `~~${selection}~~` : '~~~~';
                pos.ch = selection ? pos.ch : pos.ch + 2;
            }
            if (type === 'underline') {
                value = selection ? `<u>${selection}</u>` : '<u></u>';
                pos.ch = selection ? pos.ch : pos.ch + 3;
            }
            if (type === 'olist') {
                value = selection ? `- ${selection}` : '- ';
                pos.ch = selection ? pos.ch : pos.ch + 2;
            }
            if (type === 'ulist') {
                value = selection ? `1. ${selection}` : '1. ';
                pos.ch = selection ? pos.ch : pos.ch + 3;
            }
            if (type === 'todo') {
                value = selection ? `- [ ] ${selection}` : '- [ ] ';
                pos.ch = selection ? pos.ch : pos.ch + 6;
            }
            if (type === 'link') {
                value = '[连接说明](连接地址 "连接标题")';
            }
            if (type === 'image') {
                value = selection ? `${selection} ![](图片地址 "图片描述")` : '![图片描述](图片地址 "图片描述")\n';
            }
            if (type === 'quote') {
                value = selection ? `> ${selection}` : '> ';
                pos.ch = selection ? pos.ch : pos.ch + 2;
            }
            this.CodeMirror.editor.replaceSelection(value);
            this.CodeMirror.editor.focus();
            this.CodeMirror.editor.setCursor(pos.line, pos.ch);
        };
        this.state = {
            preview: true,
        };
    }
    render() {
        const _a = this.props, { prefixCls, className, toolbars, onChange } = _a, codemirrorProps = __rest(_a, ["prefixCls", "className", "toolbars", "onChange"]);
        return (React.createElement("div", { className: classnames_1.default(prefixCls, className) },
            React.createElement(ToolBar_1.default, { toolbars: toolbars, onClick: this.onClick }),
            React.createElement("div", { className: classnames_1.default(`${prefixCls}-content`) },
                React.createElement(CodeMirror_1.default, Object.assign({ style: { width: this.state.preview ? '50%' : '100%' }, width: this.state.preview ? '50%' : '100%', ref: this.getInstance }, codemirrorProps, { onChange: this.onChange })),
                React.createElement(PreviewMarkdown_1.default, { ref: (pmd) => this.preview = pmd, value: this.props.value, visble: this.state.preview }))));
    }
    previewMarkdown() {
        this.setState({ preview: !this.state.preview });
    }
}
MarkdownEditor.displayName = 'MarkdownEditor';
MarkdownEditor.defaultProps = {
    onChange: () => null,
    prefixCls: 'md-editor',
    value: '',
};
exports.default = MarkdownEditor;
